local _, addon = ...

-- functions
local format = string.format
local C_ChallengeMode = C_ChallengeMode
local C_ChallengeMode_RequestMapInfo = C_ChallengeMode.RequestMapInfo
local C_ChallengeMode_RequestRewards = C_ChallengeMode.RequestRewards
local C_ChallengeMode_GetMapPlayerStats = C_ChallengeMode.GetMapPlayerStats
local C_ChallengeMode_GetMapTable = C_ChallengeMode.GetMapTable
local C_Timer = C_Timer
local C_Timer_NewTimer = C_Timer.NewTimer
local CreateFrame = CreateFrame
-- constants
-- frames
-- libs

addon:RegisterCallback(addon.EVENT_RECORD_SCAN_STARTED, function()
    addon:Debug("Scanning for character completion records...")
end)

addon:RegisterCallback(addon.EVENT_RECORD_SCAN_FINISHED, function(_, mapId, level)
    if mapId then
        addon:Debug(format("Character has completed %s %d.", addon:GetMapName(mapId), level))
    else
        addon:Debug("No record found.")
    end

    addon:SetCharacterRecord(addon:GetCharacterKey(), mapId, level)
end)

do
    local scanDelay = 3
    local scanTimer
    local function scanCallback()
        addon:Fire(addon.EVENT_RECORD_SCAN_STARTED)

        local recordMapId, recordLevel = nil, 0

        local maps = C_ChallengeMode_GetMapTable()
        for _, mapId in pairs(maps) do
            local _, _, bestLevel = C_ChallengeMode_GetMapPlayerStats(mapId)

            if bestLevel and bestLevel > recordLevel then
                recordMapId = mapId
                recordLevel = bestLevel
            end
        end

        addon:Fire(addon.EVENT_RECORD_SCAN_FINISHED, recordMapId, recordLevel)
    end

    local function onEvent(_, event)
        if event == "PLAYER_ENTERING_WORLD" then
            C_ChallengeMode_RequestMapInfo()
            C_ChallengeMode_RequestRewards()
            return
        end

        if scanTimer then
            scanTimer:Cancel()
        end

        scanTimer = C_Timer_NewTimer(scanDelay, scanCallback)
    end

    local f = CreateFrame("frame")
    f:SetScript("OnEvent", onEvent)
    f:RegisterEvent("PLAYER_ENTERING_WORLD")
    f:RegisterEvent("CHALLENGE_MODE_LEADERS_UPDATE")
    f:RegisterEvent("CHALLENGE_MODE_MAPS_UPDATE")
end
